<?php

namespace Drupal\ai_auto_reference\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * The AI Autoreference configurable settings.
 *
 * @package Drupal\ai_auto_reference\Form
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'ai_auto_reference.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_auto_reference_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ai_auto_reference.settings');

    $form['service'] = [
      '#type' => 'select',
      '#title' => $this->t('AI API service'),
      '#description' => $this->t('This is the AI API service to use. Currently only ChatGPT is supported.'),
      '#options' => [
        'gpt' => $this->t('ChatGPT'),
      ],
      '#default_value' => $config->get('service'),
    ];
    $form['service_model'] = [
      '#type' => 'select',
      '#title' => $this->t('AI API service model'),
      '#description' => $this->t('This is the model within the AI API service to use.'),
      '#options' => [
        'gpt-3.5-turbo' => $this->t('gpt-3.5-turbo'),
        'gpt-4' => $this->t('gpt-4'),
        'gpt-4-1106-preview' => $this->t('gpt-4-1106-preview'),
        'gpt-4-turbo' => $this->t('gpt-4-turbo'),
      ],
      '#default_value' => $config->get('service_model'),
    ];
    $form['service_api_key'] = [
      '#type' => 'password',
      '#title' => $this->t('AI API key'),
      '#description' => $this->t('This is the API key for the selected service. Leave blank to leave unchanged.'),
      '#default_value' => '',
    ];
    $form['token_limit'] = [
      '#type' => 'number',
      '#step' => 1,
      '#title' => $this->t('Token Limit'),
      '#description' => $this->t('The number of tokens to limit API calls to.'),
      '#default_value' => $config->get('token_limit'),
    ];
    $form['auto_apply_suggestions'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Auto-apply suggestions'),
      '#description' => $this->t('By default, editors will be shown a screen with the suggested relationships, where suggestions can be optionally skipped then saved. Check this box to automatically accept the suggestions from the AI.'),
      '#default_value' => $config->get('auto_apply_suggestions'),
    ];
    $form['auto_apply_relevance_levels'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Auto-apply relevance levels'),
      '#description' => $this->t('By default, editors will be shown a screen with the suggested relationships, where suggestions can be optionally skipped then saved. Select which  relevance levels you would like to automatically apply.'),
      '#options' => [
        'high' => $this->t('High Relevance'),
        'medium' => $this->t('Low Relevance'),
      ],
      '#multiple' => TRUE,
      '#default_value' => $config->get('auto_apply_relevance_levels'),
      '#states' => [
        'visible' => [
          ':input[name="auto_apply_suggestions"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="auto_apply_suggestions"]' => ['checked' => TRUE],
        ],
      ],
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Each model has its token limits.
    $limits = [
      'gpt-3.5-turbo' => 4000,
      'gpt-4' => 8000,
      'gpt-4-1106-preview' => 128000,
      'gpt-4-turbo' => 128000,
    ];
    $model = $form_state->getValue('service_model');
    $token_limit = $form_state->getValue('token_limit');
    if ($token_limit > $limits[$model]) {
      $form_state->setErrorByName('token_limit', $this->t('Maximum token limit for selected model is %limit.', ['%limit' => $limits[$model]]));
    }

    // Ensure at least one of relevance level is applied when
    // auto apply suggestions is in place.
    if (
      $form_state->getValue('auto_apply_suggestions')
      && !array_filter($form_state->getValue('auto_apply_relevance_levels'))
    ) {
      $message = $this->t('At least one level of relevance should be auto-applied.');
      $form_state->setErrorByName('auto_apply_relevance_levels', $message);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $relevance_levels = array_filter($form_state->getValue('auto_apply_relevance_levels'));
    $config = $this->config('ai_auto_reference.settings')
      ->set('service', $form_state->getValue('service'))
      ->set('service_model', $form_state->getValue('service_model'))
      ->set('token_limit', $form_state->getValue('token_limit'))
      ->set('auto_apply_suggestions', $form_state->getValue('auto_apply_suggestions'))
      ->set('auto_apply_relevance_levels', array_keys($relevance_levels));

    // Only save API key if there is a new value, otherwise leave
    // old value unchanged.
    $service_api_key = $form_state->getValue('service_api_key');
    if (!empty($service_api_key)) {
      $config->set('service_api_key', $service_api_key);
    }

    $config->save();
  }

}
